/* cmdsocket.h */

int cmdsocket_init(void);
void cmdsocket_close(void);
int cmdsocket_listen(int port);
int cmdsocket_accept(void);
int cmdsocket_read(void);
int cmdsocket_write(char *buffer, int len, int off);
int cmdsocket_prefix(char *);
